double *lightcurve(int n_layers, int n_points, double *t, double tc, double per, double a, double inc, double ecc, double omega, double r_s, double r2,double xi,double T_n,double delta_T,double u1, double u2,double T_s);
double *call_blocked(int n_layers, int n_points, double *x2, double *y2, double r2);