double **generate_planet(int n_layers);
void map_model(double **planet,int n_layers,double xi, double T_n, double delta_T,double lambda0, double phi0, double u1, double u2);
double zhang_2016(double lat, double lon, double zeta, double T_n, double delta_T);