double find_segment_area(double *c1, double *c2,double x2,double y2,double r2);
double segment(double r, double theta);
double sector(double r, double theta);
double find_sector_region(double r1, double r2, double theta);
double find_circles_region(double x1, double y1, double r1, double x2, double y2, double r2);